@@ -42,10 +42,8 @@
 
 #include "test_precomp.hpp"
 
-using namespace cv;
+namespace opencv_test { namespace {
 
-namespace cvtest
-{
     class CV_BilateralFilterTest :
         public cvtest::BaseTest
     {
@@ -128,7 +126,8 @@ namespace cvtest
         d = radius*2 + 1;
         // compute the min/max range for the input image (even if multichannel)
 
-        minMaxLoc( src.reshape(1), &minValSrc, &maxValSrc );
+        // TODO cvtest
+        cv::minMaxLoc( src.reshape(1), &minValSrc, &maxValSrc );
         if(std::abs(minValSrc - maxValSrc) < FLT_EPSILON)
         {
             src.copyTo(dst);
@@ -137,8 +136,8 @@ namespace cvtest
 
         // temporary copy of the image with borders for easy processing
         Mat temp;
-        copyMakeBorder( src, temp, radius, radius, radius, radius, borderType );
-        patchNaNs(temp);
+        cv::copyMakeBorder( src, temp, radius, radius, radius, radius, borderType );
+        cv::patchNaNs(temp);
 
         // allocate lookup tables
         vector<float> _space_weight(d*d);
@@ -204,7 +203,7 @@ namespace cvtest
             }
             else
             {
-                assert( cn == 3 );
+                CV_Assert( cn == 3 );
                 for( j = 0; j < size.width*3; j += 3 )
                 {
                     float sum_b = 0, sum_g = 0, sum_r = 0, wsum = 0;
@@ -244,27 +243,30 @@ namespace cvtest
 
         rng.fill(_src, RNG::UNIFORM, 0, 256);
 
-        _sigma_color = _sigma_space = 1.;
+        _sigma_color = _sigma_space = rng.uniform(0., 10.);
 
         return 1;
     }
 
     int CV_BilateralFilterTest::validate_test_results(int test_case_index)
     {
-        static const double eps = 4;
-
+        double eps = (_src.depth() < CV_32F)?1:5e-3;
+        double e;
         Mat reference_dst, reference_src;
         if (_src.depth() == CV_32F)
+        {
             reference_bilateral_filter(_src, reference_dst, _d, _sigma_color, _sigma_space);
+            e = cvtest::norm(reference_dst, _parallel_dst, NORM_INF|NORM_RELATIVE);
+        }
         else
         {
             int type = _src.type();
             _src.convertTo(reference_src, CV_32F);
             reference_bilateral_filter(reference_src, reference_dst, _d, _sigma_color, _sigma_space);
             reference_dst.convertTo(reference_dst, type);
+            e = cvtest::norm(reference_dst, _parallel_dst, NORM_INF);
         }
 
-        double e = cvtest::norm(reference_dst, _parallel_dst, NORM_L2);
         if (e > eps)
         {
             ts->printf(cvtest::TS::CONSOLE, "actual error: %g, expected: %g", e, eps);
@@ -287,4 +289,4 @@ namespace cvtest
         test.safe_run();
     }
 
-} // end of namespace cvtest
+}} // namespace
