@@ -43,8 +43,9 @@ testrunner.options.maxBlockDuration = 20000; // cause opencv_js.js need time to
 
 testrunner.run(
     {
-        code: 'opencv.js',
-        tests: ['test_mat.js',
+        code: {path: "opencv.js", namespace: "cv"},
+        tests: ['init_cv.js',
+                'test_mat.js',
                 'test_utils.js',
                 'test_core.js',
                 'test_imgproc.js',
@@ -57,7 +58,10 @@ testrunner.run(
     },
     function(err, report) {
         console.log(report.failed + ' failed, ' + report.passed + ' passed');
-        if (report.failed) {
+        if (report.failed || err) {
+            if (err) {
+                console.log(err);
+            }
             process.on('exit', function() {
                 process.exit(1);
             });
