@@ -1,6 +1,7 @@
 from typing import Sequence, Generator, Tuple, Optional, Union
 import weakref
 import abc
+from itertools import chain
 
 from .node import ASTNode, ASTNodeType
 
@@ -30,8 +31,9 @@ class TypeNode(abc.ABC):
     "typing.Optional[Size]"
     """
 
-    def __init__(self, ctype_name: str) -> None:
+    def __init__(self, ctype_name: str, required_modules: Tuple[str, ...] = ()) -> None:
         self.ctype_name = ctype_name
+        self._required_modules = required_modules
 
     @abc.abstractproperty
     def typename(self) -> str:
@@ -113,6 +115,10 @@ class TypeNode(abc.ABC):
         """
         yield from ()
 
+    @property
+    def required_modules(self) -> Tuple[str, ...]:
+        return self._required_modules
+
     @property
     def is_resolved(self) -> bool:
         return True
@@ -163,18 +169,20 @@ class AnyTypeNode(TypeNode):
     """
     @property
     def typename(self) -> str:
-        return "typing.Any"
+        return "_typing.Any"
 
     @property
     def required_usage_imports(self) -> Generator[str, None, None]:
-        yield "import typing"
+        yield "import typing as _typing"
 
 
 class PrimitiveTypeNode(TypeNode):
     """Type node representing a primitive built-in types e.g. int, float, str.
     """
-    def __init__(self, ctype_name: str, typename: Optional[str] = None) -> None:
-        super().__init__(ctype_name)
+    def __init__(self, ctype_name: str,
+                 typename: Optional[str] = None,
+                 required_modules: Tuple[str, ...] = ()) -> None:
+        super().__init__(ctype_name, required_modules)
         self._typename = typename if typename is not None else ctype_name
 
     @property
@@ -182,28 +190,32 @@ class PrimitiveTypeNode(TypeNode):
         return self._typename
 
     @classmethod
-    def int_(cls, ctype_name: Optional[str] = None):
+    def int_(cls, ctype_name: Optional[str] = None,
+             required_modules: Tuple[str, ...] = ()):
         if ctype_name is None:
             ctype_name = "int"
-        return PrimitiveTypeNode(ctype_name, typename="int")
+        return PrimitiveTypeNode(ctype_name, typename="int", required_modules=required_modules)
 
     @classmethod
-    def float_(cls, ctype_name: Optional[str] = None):
+    def float_(cls, ctype_name: Optional[str] = None,
+               required_modules: Tuple[str, ...] = ()):
         if ctype_name is None:
             ctype_name = "float"
-        return PrimitiveTypeNode(ctype_name, typename="float")
+        return PrimitiveTypeNode(ctype_name, typename="float", required_modules=required_modules)
 
     @classmethod
-    def bool_(cls, ctype_name: Optional[str] = None):
+    def bool_(cls, ctype_name: Optional[str] = None,
+              required_modules: Tuple[str, ...] = ()):
         if ctype_name is None:
             ctype_name = "bool"
-        return PrimitiveTypeNode(ctype_name, typename="bool")
+        return PrimitiveTypeNode(ctype_name, typename="bool", required_modules=required_modules)
 
     @classmethod
-    def str_(cls, ctype_name: Optional[str] = None):
+    def str_(cls, ctype_name: Optional[str] = None,
+             required_modules: Tuple[str, ...] = ()):
         if ctype_name is None:
             ctype_name = "string"
-        return PrimitiveTypeNode(ctype_name, "str")
+        return PrimitiveTypeNode(ctype_name, "str", required_modules=required_modules)
 
 
 class AliasRefTypeNode(TypeNode):
@@ -224,8 +236,9 @@ class AliasRefTypeNode(TypeNode):
     ```
     """
     def __init__(self, alias_ctype_name: str,
-                 alias_export_name: Optional[str] = None):
-        super().__init__(alias_ctype_name)
+                 alias_export_name: Optional[str] = None,
+                 required_modules: Tuple[str, ...] = ()):
+        super().__init__(alias_ctype_name, required_modules)
         if alias_export_name is None:
             self.alias_export_name = alias_ctype_name
         else:
@@ -258,8 +271,9 @@ class AliasTypeNode(TypeNode):
     """
     def __init__(self, ctype_name: str, value: TypeNode,
                  export_name: Optional[str] = None,
-                 doc: Optional[str] = None) -> None:
-        super().__init__(ctype_name)
+                 doc: Optional[str] = None,
+                 required_modules: Tuple[str, ...] = ()) -> None:
+        super().__init__(ctype_name, required_modules)
         self.value = value
         self._export_name = export_name
         self.doc = doc
@@ -298,20 +312,21 @@ class AliasTypeNode(TypeNode):
 
     @classmethod
     def int_(cls, ctype_name: str, export_name: Optional[str] = None,
-             doc: Optional[str] = None):
-        return cls(ctype_name, PrimitiveTypeNode.int_(), export_name, doc)
+             doc: Optional[str] = None, required_modules: Tuple[str, ...] = ()):
+        return cls(ctype_name, PrimitiveTypeNode.int_(), export_name, doc, required_modules)
 
     @classmethod
     def float_(cls, ctype_name: str, export_name: Optional[str] = None,
-               doc: Optional[str] = None):
-        return cls(ctype_name, PrimitiveTypeNode.float_(), export_name, doc)
+               doc: Optional[str] = None, required_modules: Tuple[str, ...] = ()):
+        return cls(ctype_name, PrimitiveTypeNode.float_(), export_name, doc, required_modules)
 
     @classmethod
     def array_ref_(cls, ctype_name: str, array_ref_name: str,
                    shape: Optional[Tuple[int, ...]],
                    dtype: Optional[str] = None,
                    export_name: Optional[str] = None,
-                   doc: Optional[str] = None):
+                   doc: Optional[str] = None,
+                   required_modules: Tuple[str, ...] = ()):
         """Create alias to array reference alias `array_ref_name`.
 
         This is required to preserve backward compatibility with Python < 3.9
@@ -332,72 +347,84 @@ class AliasTypeNode(TypeNode):
         else:
             doc += f". NDArray(shape={shape}, dtype={dtype})"
         return cls(ctype_name, AliasRefTypeNode(array_ref_name),
-                   export_name, doc)
+                   export_name, doc, required_modules)
 
     @classmethod
     def union_(cls, ctype_name: str, items: Tuple[TypeNode, ...],
                export_name: Optional[str] = None,
-               doc: Optional[str] = None):
+               doc: Optional[str] = None,
+               required_modules: Tuple[str, ...] = ()):
         return cls(ctype_name, UnionTypeNode(ctype_name, items),
-                   export_name, doc)
+                   export_name, doc, required_modules)
 
     @classmethod
     def optional_(cls, ctype_name: str, item: TypeNode,
                   export_name: Optional[str] = None,
-                  doc: Optional[str] = None):
-        return cls(ctype_name, OptionalTypeNode(item), export_name, doc)
+                  doc: Optional[str] = None,
+                  required_modules: Tuple[str, ...] = ()):
+        return cls(ctype_name, OptionalTypeNode(item), export_name, doc, required_modules)
 
     @classmethod
     def sequence_(cls, ctype_name: str, item: TypeNode,
                   export_name: Optional[str] = None,
-                  doc: Optional[str] = None):
+                  doc: Optional[str] = None,
+                  required_modules: Tuple[str, ...] = ()):
         return cls(ctype_name, SequenceTypeNode(ctype_name, item),
-                   export_name, doc)
+                   export_name, doc, required_modules)
 
     @classmethod
     def tuple_(cls, ctype_name: str, items: Tuple[TypeNode, ...],
                export_name: Optional[str] = None,
-               doc: Optional[str] = None):
+               doc: Optional[str] = None,
+               required_modules: Tuple[str, ...] = ()):
         return cls(ctype_name, TupleTypeNode(ctype_name, items),
-                   export_name, doc)
+                   export_name, doc, required_modules)
 
     @classmethod
     def class_(cls, ctype_name: str, class_name: str,
                export_name: Optional[str] = None,
-               doc: Optional[str] = None):
+               doc: Optional[str] = None,
+               required_modules: Tuple[str, ...] = ()):
         return cls(ctype_name, ASTNodeTypeNode(class_name),
-                   export_name, doc)
+                   export_name, doc, required_modules)
 
     @classmethod
     def callable_(cls, ctype_name: str,
                   arg_types: Union[TypeNode, Sequence[TypeNode]],
                   ret_type: TypeNode = NoneTypeNode("void"),
                   export_name: Optional[str] = None,
-                  doc: Optional[str] = None):
+                  doc: Optional[str] = None,
+                  required_modules: Tuple[str, ...] = ()):
         return cls(ctype_name,
                    CallableTypeNode(ctype_name, arg_types, ret_type),
-                   export_name, doc)
+                   export_name, doc, required_modules)
 
     @classmethod
     def ref_(cls, ctype_name: str, alias_ctype_name: str,
              alias_export_name: Optional[str] = None,
-             export_name: Optional[str] = None, doc: Optional[str] = None):
+             export_name: Optional[str] = None,
+             doc: Optional[str] = None,
+             required_modules: Tuple[str, ...] = ()):
         return cls(ctype_name,
                    AliasRefTypeNode(alias_ctype_name, alias_export_name),
-                   export_name, doc)
+                   export_name, doc, required_modules)
 
     @classmethod
     def dict_(cls, ctype_name: str, key_type: TypeNode, value_type: TypeNode,
-              export_name: Optional[str] = None, doc: Optional[str] = None):
+              export_name: Optional[str] = None, doc: Optional[str] = None,
+              required_modules: Tuple[str, ...] = ()):
         return cls(ctype_name, DictTypeNode(ctype_name, key_type, value_type),
-                   export_name, doc)
+                   export_name, doc, required_modules)
 
 
 class ConditionalAliasTypeNode(TypeNode):
     """Type node representing an alias protected by condition checked in runtime.
+    For typing-related conditions, prefer using typing.TYPE_CHECKING. For a full explanation, see:
+    https://github.com/opencv/opencv/pull/23927#discussion_r1256326835
+
     Example:
     ```python
-    if numpy.lib.NumpyVersion(numpy.__version__) > "1.20.0" and sys.version_info >= (3, 9)
+    if typing.TYPE_CHECKING
         NumPyArray = numpy.ndarray[typing.Any, numpy.dtype[numpy.generic]]
     else:
         NumPyArray = numpy.ndarray
@@ -407,10 +434,10 @@ class ConditionalAliasTypeNode(TypeNode):
 
     ConditionalAliasTypeNode(
         "NumPyArray",
-        'numpy.lib.NumpyVersion(numpy.__version__) > "1.20.0" and sys.version_info >= (3, 9)',
+        'typing.TYPE_CHECKING',
         NDArrayTypeNode("NumPyArray"),
         NDArrayTypeNode("NumPyArray", use_numpy_generics=False),
-        condition_required_imports=("import numpy", "import sys")
+        condition_required_imports=("import typing",)
     )
     ```
     """
@@ -448,6 +475,11 @@ class ConditionalAliasTypeNode(TypeNode):
     def required_usage_imports(self) -> Generator[str, None, None]:
         yield "import cv2.typing"
 
+    @property
+    def required_modules(self) -> Tuple[str, ...]:
+        return (*self.positive_branch_type.required_modules,
+                *self.negative_branch_type.required_modules)
+
     @property
     def is_resolved(self) -> bool:
         return self.positive_branch_type.is_resolved \
@@ -468,14 +500,14 @@ class ConditionalAliasTypeNode(TypeNode):
     def numpy_array_(cls, ctype_name: str, export_name: Optional[str] = None,
                      shape: Optional[Tuple[int, ...]] = None,
                      dtype: Optional[str] = None):
+        """Type subscription is not possible in python 3.8 and older numpy versions."""
         return cls(
             ctype_name,
-            ('numpy.lib.NumpyVersion(numpy.__version__) > "1.20.0" '
-             'and sys.version_info >= (3, 9)'),
+            "_typing.TYPE_CHECKING",
             NDArrayTypeNode(ctype_name, shape, dtype),
             NDArrayTypeNode(ctype_name, shape, dtype,
                             use_numpy_generics=False),
-            condition_required_imports=("import numpy", "import sys")
+            condition_required_imports=("import typing as _typing",)
         )
 
 
@@ -496,14 +528,14 @@ class NDArrayTypeNode(TypeNode):
         if self._use_numpy_generics:
             # NOTE: Shape is not fully supported yet
             dtype = self.dtype if self.dtype is not None else "numpy.generic"
-            return f"numpy.ndarray[typing.Any, numpy.dtype[{dtype}]]"
+            return f"numpy.ndarray[_typing.Any, numpy.dtype[{dtype}]]"
         return "numpy.ndarray"
 
     @property
     def required_usage_imports(self) -> Generator[str, None, None]:
         yield "import numpy"
         # if self.shape is None:
-        yield "import typing"
+        yield "import typing as _typing"
 
 
 class ASTNodeTypeNode(TypeNode):
@@ -516,8 +548,9 @@ class ASTNodeTypeNode(TypeNode):
     too expensive and error prone.
     """
     def __init__(self, ctype_name: str, typename: Optional[str] = None,
-                 module_name: Optional[str] = None) -> None:
-        super().__init__(ctype_name)
+                 module_name: Optional[str] = None,
+                 required_modules: Tuple[str, ...] = ()) -> None:
+        super().__init__(ctype_name, required_modules)
         self._typename = typename if typename is not None else ctype_name
         self._module_name = module_name
         self._ast_node: Optional[weakref.ProxyType[ASTNode]] = None
@@ -604,14 +637,20 @@ class ASTNodeTypeNode(TypeNode):
 class AggregatedTypeNode(TypeNode):
     """Base type node for type nodes representing an aggregation of another
     type nodes e.g. tuple, sequence or callable."""
-    def __init__(self, ctype_name: str, items: Sequence[TypeNode]) -> None:
-        super().__init__(ctype_name)
+    def __init__(self, ctype_name: str, items: Sequence[TypeNode],
+                 required_modules: Tuple[str, ...] = ()) -> None:
+        super().__init__(ctype_name, required_modules)
         self.items = list(items)
 
     @property
     def is_resolved(self) -> bool:
         return all(item.is_resolved for item in self.items)
 
+    @property
+    def required_modules(self) -> Tuple[str, ...]:
+        return (*chain.from_iterable(item.required_modules for item in self.items),
+                *self._required_modules)
+
     def resolve(self, root: ASTNode) -> None:
         errors = []
         for item in filter(lambda item: not item.is_resolved, self):
@@ -665,13 +704,13 @@ class ContainerTypeNode(AggregatedTypeNode):
 
     @property
     def required_definition_imports(self) -> Generator[str, None, None]:
-        yield "import typing"
+        yield "import typing as _typing"
         yield from super().required_definition_imports
 
     @property
     def required_usage_imports(self) -> Generator[str, None, None]:
         if TypeNode.compatible_to_runtime_usage:
-            yield "import typing"
+            yield "import typing as _typing"
         yield from super().required_usage_imports
 
     @abc.abstractproperty
@@ -687,12 +726,13 @@ class SequenceTypeNode(ContainerTypeNode):
     """Type node representing a homogeneous collection of elements with
     possible unknown length.
     """
-    def __init__(self, ctype_name: str, item: TypeNode) -> None:
-        super().__init__(ctype_name, (item, ))
+    def __init__(self, ctype_name: str, item: TypeNode,
+                 required_modules: Tuple[str, ...] = ()) -> None:
+        super().__init__(ctype_name, (item, ), required_modules)
 
     @property
     def type_format(self) -> str:
-        return "typing.Sequence[{}]"
+        return "_typing.Sequence[{}]"
 
     @property
     def types_separator(self) -> str:
@@ -706,7 +746,7 @@ class TupleTypeNode(ContainerTypeNode):
     @property
     def type_format(self) -> str:
         if TypeNode.compatible_to_runtime_usage:
-            return "typing.Tuple[{}]"
+            return "_typing.Tuple[{}]"
         return "tuple[{}]"
 
     @property
@@ -720,7 +760,7 @@ class UnionTypeNode(ContainerTypeNode):
     @property
     def type_format(self) -> str:
         if TypeNode.compatible_to_runtime_usage:
-            return "typing.Union[{}]"
+            return "_typing.Union[{}]"
         return "{}"
 
     @property
@@ -734,13 +774,14 @@ class OptionalTypeNode(ContainerTypeNode):
     """Type node representing optional type which is effectively is a union
     of value type node and None.
     """
-    def __init__(self, value: TypeNode) -> None:
-        super().__init__(value.ctype_name, (value,))
+    def __init__(self, value: TypeNode,
+                 required_modules: Tuple[str, ...] = ()) -> None:
+        super().__init__(value.ctype_name, (value,), required_modules)
 
     @property
     def type_format(self) -> str:
         if TypeNode.compatible_to_runtime_usage:
-            return "typing.Optional[{}]"
+            return "_typing.Optional[{}]"
         return "{} | None"
 
     @property
@@ -752,8 +793,9 @@ class DictTypeNode(ContainerTypeNode):
     """Type node representing a homogeneous key-value mapping.
     """
     def __init__(self, ctype_name: str, key_type: TypeNode,
-                 value_type: TypeNode) -> None:
-        super().__init__(ctype_name, (key_type, value_type))
+                 value_type: TypeNode,
+                 required_modules: Tuple[str, ...] = ()) -> None:
+        super().__init__(ctype_name, (key_type, value_type), required_modules)
 
     @property
     def key_type(self) -> TypeNode:
@@ -766,7 +808,7 @@ class DictTypeNode(ContainerTypeNode):
     @property
     def type_format(self) -> str:
         if TypeNode.compatible_to_runtime_usage:
-            return "typing.Dict[{}]"
+            return "_typing.Dict[{}]"
         return "dict[{}]"
 
     @property
@@ -791,11 +833,12 @@ class CallableTypeNode(AggregatedTypeNode):
     """
     def __init__(self, ctype_name: str,
                  arg_types: Union[TypeNode, Sequence[TypeNode]],
-                 ret_type: TypeNode = NoneTypeNode("void")) -> None:
+                 ret_type: TypeNode = NoneTypeNode("void"),
+                 required_modules: Tuple[str, ...] = ()) -> None:
         if isinstance(arg_types, TypeNode):
-            super().__init__(ctype_name, (arg_types, ret_type))
+            super().__init__(ctype_name, (arg_types, ret_type), required_modules)
         else:
-            super().__init__(ctype_name, (*arg_types, ret_type))
+            super().__init__(ctype_name, (*arg_types, ret_type), required_modules)
 
     @property
     def arg_types(self) -> Sequence[TypeNode]:
@@ -807,35 +850,51 @@ class CallableTypeNode(AggregatedTypeNode):
 
     @property
     def typename(self) -> str:
-        return 'typing.Callable[[{}], {}]'.format(
+        return '_typing.Callable[[{}], {}]'.format(
             ', '.join(arg.typename for arg in self.arg_types),
             self.ret_type.typename
         )
 
     @property
     def full_typename(self) -> str:
-        return 'typing.Callable[[{}], {}]'.format(
+        return '_typing.Callable[[{}], {}]'.format(
             ', '.join(arg.full_typename for arg in self.arg_types),
             self.ret_type.full_typename
         )
 
     def relative_typename(self, module: str) -> str:
-        return 'typing.Callable[[{}], {}]'.format(
+        return '_typing.Callable[[{}], {}]'.format(
             ', '.join(arg.relative_typename(module) for arg in self.arg_types),
             self.ret_type.relative_typename(module)
         )
 
     @property
     def required_definition_imports(self) -> Generator[str, None, None]:
-        yield "import typing"
+        yield "import typing as _typing"
         yield from super().required_definition_imports
 
     @property
     def required_usage_imports(self) -> Generator[str, None, None]:
-        yield "import typing"
+        yield "import typing as _typing"
         yield from super().required_usage_imports
 
 
+class ClassTypeNode(ContainerTypeNode):
+    """Type node representing types themselves (refer to typing.Type)
+    """
+    def __init__(self, value: TypeNode,
+                 required_modules: Tuple[str, ...] = ()) -> None:
+        super().__init__(value.ctype_name, (value,), required_modules)
+
+    @property
+    def type_format(self) -> str:
+        return "_typing.Type[{}]"
+
+    @property
+    def types_separator(self) -> str:
+        return ", "
+
+
 def _resolve_symbol(root: Optional[ASTNode], full_symbol_name: str) -> Optional[ASTNode]:
     """Searches for a symbol with the given full export name in the AST
     starting from the `root`.
