@@ -3,6 +3,7 @@
 // of this distribution and at http://opencv.org/license.html.
 
 #include "test_precomp.hpp"
+#include "opencv2/core/utils/filesystem.hpp"
 
 namespace opencv_test
 {
@@ -47,6 +48,14 @@ public:
             throw SkipTestException(cv::String("Backend is not available/disabled: ") + cv::videoio_registry::getBackendName(apiPref));
         if (cvtest::skipUnstableTests && apiPref == CAP_MSMF && (ext == "h264" || ext == "h265" || ext == "mpg"))
             throw SkipTestException("Unstable MSMF test");
+#ifdef __linux__
+        if (cvtest::skipUnstableTests && apiPref == CAP_GSTREAMER &&
+            (ext == "avi" || ext == "mkv") &&
+            (video_file.find("MPEG") != std::string::npos))
+        {
+            throw SkipTestException("Unstable GSTREAMER test");
+        }
+#endif
         writeVideo();
         VideoCapture cap;
         ASSERT_NO_THROW(cap.open(video_file, apiPref));
@@ -67,6 +76,11 @@ public:
             std::cout << "CAP_PROP_FRAME_COUNT is not supported by backend. Assume 50 frames." << std::endl;
             n_frames = 50;
         }
+        // GStreamer can't read frame count of big_buck_bunny.wmv
+        if (apiPref == CAP_GSTREAMER && ext == "wmv")
+        {
+            n_frames = 125;
+        }
 
         {
             SCOPED_TRACE("consecutive read");
@@ -166,7 +180,7 @@ public:
         EXPECT_NO_THROW(count_prop = (int)cap.get(CAP_PROP_FRAME_COUNT));
         // mpg file reports 5.08 sec * 24 fps => property returns 122 frames
         // but actual number of frames returned is 125
-        if (ext != "mpg")
+        if (ext != "mpg" && !(apiPref == CAP_GSTREAMER && ext == "wmv"))
         {
             if (count_prop > 0)
             {
@@ -200,12 +214,11 @@ public:
         if (!isBackendAvailable(apiPref, cv::videoio_registry::getStreamBackends()))
             throw SkipTestException(cv::String("Backend is not available/disabled: ") + cv::videoio_registry::getBackendName(apiPref));
 
-        // GStreamer: https://github.com/opencv/opencv/issues/19025
-        if (apiPref == CAP_GSTREAMER)
+        if (((apiPref == CAP_GSTREAMER) && (ext == "avi")))
             throw SkipTestException(cv::String("Backend ") +  cv::videoio_registry::getBackendName(apiPref) +
-                    cv::String(" does not return reliable values for CAP_PROP_POS_MSEC property"));
+                    cv::String(" does not support CAP_PROP_POS_MSEC option"));
 
-        if (((apiPref == CAP_FFMPEG) && ((ext == "h264") || (ext == "h265"))))
+        if (((apiPref == CAP_FFMPEG || apiPref == CAP_GSTREAMER) && ((ext == "h264") || (ext == "h265"))))
             throw SkipTestException(cv::String("Backend ") +  cv::videoio_registry::getBackendName(apiPref) +
                     cv::String(" does not support CAP_PROP_POS_MSEC option"));
 
@@ -221,8 +234,8 @@ public:
         // mpg file reports 5.08 sec * 24 fps => property returns 122 frames,but actual number of frames returned is 125
         // HACK: CAP_PROP_FRAME_COUNT is not supported for vmw + MSMF. Just force check for all 125 frames
         if (ext == "mpg")
-            EXPECT_GT(frame_count, 121);
-        else if ((ext == "wmv") && (apiPref == CAP_MSMF))
+            EXPECT_GE(frame_count, 114);
+        else if ((ext == "wmv") && (apiPref == CAP_MSMF || apiPref == CAP_GSTREAMER))
             frame_count = 125;
         else
             EXPECT_EQ(frame_count, 125);
@@ -240,6 +253,9 @@ public:
             if (cvtest::debugLevel > 0)
                 std::cout << "i = " << i << ": timestamp = " << timestamp << std::endl;
             const double frame_period = 1000.f/bunny_param.getFps();
+            // big_buck_bunny.mpg starts at 0.500 msec
+            if ((ext == "mpg") && (apiPref == CAP_GSTREAMER))
+                timestamp -= 500.0;
             // NOTE: eps == frame_period, because videoCapture returns frame beginning timestamp or frame end
             // timestamp depending on codec and back-end. So the first frame has timestamp 0 or frame_period.
             EXPECT_NEAR(timestamp, i*frame_period, frame_period) << "i=" << i;
@@ -388,6 +404,7 @@ static Ext_Fourcc_PSNR synthetic_params[] = {
     {"wmv", "WMV3", 30.f, CAP_MSMF},
     {"wmv", "WVC1", 30.f, CAP_MSMF},
     {"mov", "H264", 30.f, CAP_MSMF},
+ // {"mov", "HEVC", 30.f, CAP_MSMF},  // excluded due to CI issue: https://github.com/opencv/opencv/pull/23172
 #endif
 
 #ifdef HAVE_AVFOUNDATION
@@ -844,6 +861,13 @@ TEST_P(videowriter_acceleration, write)
     std::string backend_name = cv::videoio_registry::getBackendName(backend);
     if (!videoio_registry::hasBackend(backend))
         throw SkipTestException(cv::String("Backend is not available/disabled: ") + backend_name);
+#ifdef __linux__
+    if (cvtest::skipUnstableTests && backend == CAP_GSTREAMER &&
+        (extension == "mkv") && (codecid == "MPEG"))
+    {
+        throw SkipTestException("Unstable GSTREAMER test");
+    }
+#endif
 
     const Size sz(640, 480);
     const int frameNum = 15;
@@ -991,6 +1015,7 @@ static Ext_Fourcc_PSNR hw_codecs[] = {
 #ifdef _WIN32
         {"mp4", "MPEG", 29.f, CAP_MSMF},
         {"mp4", "H264", 29.f, CAP_MSMF},
+        {"mp4", "HEVC", 29.f, CAP_MSMF},
 #endif
 };
 
@@ -1000,4 +1025,133 @@ INSTANTIATE_TEST_CASE_P(videoio, videowriter_acceleration, testing::Combine(
         testing::ValuesIn(hw_use_umat)
 ));
 
+class BufferStream : public cv::IStreamReader
+{
+public:
+    BufferStream(const std::string& filename)
+    {
+        Ptr<std::filebuf> file = makePtr<std::filebuf>();
+        file->open(filename.c_str(), std::ios::in | std::ios::binary);
+        stream = file;
+    }
+
+    BufferStream(const Ptr<std::stringbuf>& _stream) : stream(_stream) {}
+
+    long long read(char* buffer, long long size) CV_OVERRIDE
+    {
+        auto result = stream->sgetn(buffer, size);
+        return result;
+    }
+
+    long long seek(long long offset, int way) CV_OVERRIDE
+    {
+        auto result = stream->pubseekoff(offset, way == SEEK_SET ? std::ios_base::beg : (way == SEEK_END ? std::ios_base::end : std::ios_base::cur));
+        return result;
+    }
+
+private:
+    Ptr<std::streambuf> stream;
+};
+
+typedef testing::TestWithParam<tuple<std::string, VideoCaptureAPIs>> stream_capture;
+TEST_P(stream_capture, read)
+{
+    std::string ext = get<0>(GetParam());
+    VideoCaptureAPIs apiPref = get<1>(GetParam());
+    std::vector<VideoCaptureAPIs> supportedAPIs = videoio_registry::getStreamBufferedBackends();
+    if (!videoio_registry::hasBackend(apiPref))
+        throw SkipTestException(cv::String("Backend is not available/disabled: ") + cv::videoio_registry::getBackendName(apiPref));
+    if (std::find(supportedAPIs.begin(), supportedAPIs.end(), apiPref) == supportedAPIs.end())
+        throw SkipTestException(cv::String("Backend is not supported: ") + cv::videoio_registry::getBackendName(apiPref));
+    if (cvtest::skipUnstableTests && apiPref == CAP_MSMF && (ext == "h264" || ext == "h265" || ext == "mpg"))
+        throw SkipTestException("Unstable MSMF test");
+
+    if (!videoio_registry::isBackendBuiltIn(apiPref))
+    {
+        int pluginABI, pluginAPI;
+        videoio_registry::getStreamBufferedBackendPluginVersion(apiPref, pluginABI, pluginAPI);
+        if (pluginABI < 1 || (pluginABI == 1 && pluginAPI < 2))
+            throw SkipTestException(format("Buffer capture supported since ABI/API = 1/2. %s plugin is %d/%d",
+                                           cv::videoio_registry::getBackendName(apiPref).c_str(), pluginABI, pluginAPI));
+    }
+
+    VideoCapture cap;
+    String video_file = BunnyParameters::getFilename(String(".") + ext);
+    ASSERT_TRUE(utils::fs::exists(video_file));
+    EXPECT_NO_THROW(cap.open(makePtr<BufferStream>(video_file), apiPref, {}));
+    ASSERT_TRUE(cap.isOpened());
+
+    const int numFrames = 10;
+    Mat frames[numFrames];
+    Mat hardCopies[numFrames];
+    for(int i = 0; i < numFrames; i++)
+    {
+        ASSERT_NO_THROW(cap >> frames[i]);
+        EXPECT_FALSE(frames[i].empty());
+        hardCopies[i] = frames[i].clone();
+    }
+
+    for(int i = 0; i < numFrames; i++)
+        EXPECT_EQ(0, cv::norm(frames[i], hardCopies[i], NORM_INF)) << i;
+}
+INSTANTIATE_TEST_CASE_P(videoio, stream_capture,
+                          testing::Combine(
+                              testing::ValuesIn(bunny_params),
+                              testing::ValuesIn(backend_params)));
+
+// This test for stream input for container format (See test_ffmpeg/videoio_container.read test)
+typedef testing::TestWithParam<std::string> stream_capture_ffmpeg;
+TEST_P(stream_capture_ffmpeg, raw)
+{
+    std::string ext = GetParam();
+    VideoCaptureAPIs apiPref = CAP_FFMPEG;
+    std::vector<VideoCaptureAPIs> supportedAPIs = videoio_registry::getStreamBufferedBackends();
+    if (!videoio_registry::hasBackend(apiPref))
+        throw SkipTestException(cv::String("Backend is not available/disabled: ") + cv::videoio_registry::getBackendName(apiPref));
+    if (std::find(supportedAPIs.begin(), supportedAPIs.end(), apiPref) == supportedAPIs.end())
+        throw SkipTestException(cv::String("Backend is not supported: ") + cv::videoio_registry::getBackendName(apiPref));
+
+    if (!videoio_registry::isBackendBuiltIn(apiPref))
+    {
+        int pluginABI, pluginAPI;
+        videoio_registry::getStreamBufferedBackendPluginVersion(apiPref, pluginABI, pluginAPI);
+        if (pluginABI < 1 || (pluginABI == 1 && pluginAPI < 2))
+            throw SkipTestException(format("Buffer capture supported since ABI/API = 1/2. %s plugin is %d/%d",
+                                           cv::videoio_registry::getBackendName(apiPref).c_str(), pluginABI, pluginAPI));
+    }
+
+    VideoCapture container;
+    String video_file = BunnyParameters::getFilename(String(".") + ext);
+    ASSERT_TRUE(utils::fs::exists(video_file));
+    EXPECT_NO_THROW(container.open(video_file, apiPref, {CAP_PROP_FORMAT, -1}));
+    ASSERT_TRUE(container.isOpened());
+    ASSERT_EQ(-1.f, container.get(CAP_PROP_FORMAT));
+
+    auto stream = std::make_shared<std::stringbuf>();
+    Mat keyFrame;
+    while (true)
+    {
+        container >> keyFrame;
+        if (keyFrame.empty())
+            break;
+        stream->sputn(keyFrame.ptr<char>(), keyFrame.total());
+    }
+
+    VideoCapture capRef(video_file);
+    VideoCapture capStream;
+    EXPECT_NO_THROW(capStream.open(makePtr<BufferStream>(stream), apiPref, {}));
+    ASSERT_TRUE(capStream.isOpened());
+
+    const int numFrames = 10;
+    Mat frameRef, frame;
+    for (int i = 0; i < numFrames; ++i)
+    {
+        capRef >> frameRef;
+        ASSERT_NO_THROW(capStream >> frame);
+        EXPECT_FALSE(frame.empty());
+        EXPECT_EQ(0, cv::norm(frame, frameRef, NORM_INF)) << i;
+    }
+}
+INSTANTIATE_TEST_CASE_P(videoio, stream_capture_ffmpeg, testing::Values("h264", "h265", "mjpg.avi"));
+
 } // namespace
